<?php
// Author: Michael Stauber
// $Id: DatePicker.php
//
// Class to create DatePicker Elements
//

global $isPieChartDefined;
if ( $isPieChartDefined )
    return;
$isPieChartDefined = true;

include_once("uifc2/HtmlComponent.php");

class DatePicker extends HtmlComponent {

    var $BxPage;
    var $id;
    var $value;
    var $Label, $Description;
    var $i18n;
    var $access;
    var $tooltip;
    var $tooltip_placement;
    var $modus = 'all';
    var $minDate;
    var $maxDate;
    var $is_Submit;

    //
    // public methods
    //

    // constructor
    public function __construct($BxPage, $id, $value, $i18n) {
        $this->BxPage = $BxPage;
        $this->id = $id;
        $this->value = $value;
        $this->i18n = $i18n;
    }

    //
    //--- Set up Submit event on date change:
    //

    function setSubmit($var) {
        $this->is_Submit = $var;
    }

    //
    //--- Handle DatePicker modus ('hours', 'days', 'all')
    //

    function setModus($var) {
        $available_modes = array('hours', 'days', 'all');
        if (in_array($var, $available_modes)) {
            $this->modus = $var;
        }
    }

    function getModus() {
        return $this->modus;
    }

    //
    //--- Handle Min/Max Dates:
    //

    function setMinDate($value='') {
        $this->minDate = $value;
    }

    function setMaxDate($value='') {
        $this->maxDate = $value;
    }

    function getMinDate($value='') {
        return $this->minDate;
    }

    function getMaxDate($value='') {
        return $this->maxDate;
    }

    //
    //--- Tooltip generation:
    //

    function setTooltip($tooltip) {
        $this->tooltip = $tooltip;
    }

    function setTooltipPlacement($tooltip_placement) {
        $this->tooltip_placement = $tooltip_placement;
    }

    function MakeTooltip($description, $tooltip_placement = 'top') {
        $this->setTooltip($description);
        $this->setTooltipPlacement($tooltip_placement);
        $this->tooltip = 'data-toggle="tooltip" data-placement="' . $this->tooltip_placement . '" title="' . $this->tooltip . '" data-original-title="' . $this->tooltip . '" data-container="body"';
        return $this->tooltip;
    }

    function getId() {
        return $this->id;
    }

    function getValue() {
        return $this->value;
    }

    function setAccess($access) {
        $this->access = $access;
    }

    function getAccess() {
        $access = "rw";
        return $access;
    }

    function isOptional() {
        return TRUE;
    }

    // Sets the current label
    function setCurrentLabel($label) {
        $this->Label = $label;
    }

    // Returns the current label
    function getCurrentLabel() {
        if (!isset($this->Label)) {
            $this->Label = "";
        }
        return $this->Label;
    }

    // Sets the current label-description:
    function setDescription($description) {
        if (!isset($this->Description)) {
            $this->Description = "";
        }
        $this->Description = $description;
    }

    // Returns the current label-description:
    function getDescription() {
        return $this->Description;
    }

    // description: defines where the labels are placed on formfields:
    function setLabelType($labeltype) {
        $this->LabelType = $labeltype;
    }

    // Returns where the labels are placed on formfields:
    function getLabelType() {
        if (!isset($this->LabelType)) {
            $this->LabelType = "label_side top";
        }
        return $this->LabelType;
    }

    function toHtml($style = "") {

        $id = $this->getId();

        if (isset($page->BXLabel[$id])) {
            $LabelArray = array_keys($page->BXLabel[$id]);
            $HtmlLabel = $this->i18n->getHtml($LabelArray[0]);
            $DescArray = array_values($page->BXLabel[$id]);
            $HtmlDesc = $this->i18n->getWrapped($DescArray[0]);
        }

        if (!isset($HtmlLabel)) {
            $HtmlLabel = $this->getCurrentLabel();
        }

        if (!isset($HtmlDesc)) {
            $HtmlDesc = $this->getDescription();
        }

        $tooltip_out = $this->MakeTooltip($HtmlDesc, 'right');
        $label_position = '';

        if ($this->value === '') {
            $time_stamp = 'moment()';
        }

        $time_stamp_extra = '';
        if ($this->modus === 'days') {
            $format = "'YYYY-MM-DD'";
            $viewMode = "viewMode: 'days',";
            $date = new DateTime();
            $date->setTimestamp($this->value);
            $time_stamp = "'" . $date->format('Y-m-d') . "'";
            $icons = 'fa fa-calendar';
        }
        elseif ($this->modus === 'hours') {
            $format = "'LT'";
            $viewMode = "";
            $date = new DateTime();
            $date->setTimestamp($this->value);
            $time_stamp = 'new Date(\'' . $date->format('Y-m-d H:i') . '\')';
            $icons = 'fa fa-clock-o';
        }
        else {
            $format = "'YYYY-MM-DD hh:mm A'";
            $viewMode = "";
            $date = new DateTime();
            $date->setTimestamp($this->value);
            $time_stamp = "'" . $date->format('Y-m-d h:i A') . "'";
            $icons = 'fa fa-calendar';
        }

        $date_range = '';
        if (($this->minDate != '') && ($this->maxDate != '')) {
            $date_range .= 'minDate: \'' . $this->minDate . '\',' . "\n";
            $date_range .= '                        maxDate: \'' . $this->maxDate . '\',';
        }

        // Set up Submit on date change:
        $do_submit = '';
        if ($this->is_Submit != '') {
            $change_Field = $this->is_Submit;

            $do_submit = <<<HTML
            .on('dp.change', function(event) {
                                var selectedDate = event.date.format('YYYY-MM-DD');
                                $('input[name="$change_Field"]').val(selectedDate);
                                var form = $(this).closest('form.validate_form');
                                $('input[name="dateSelected"]', form).val(selectedDate);
                                form.submit();
                            });
            HTML;

        }

        $out_label =<<<HTML
            <label class="control-label mb-10 text-left $label_position" $tooltip_out>$HtmlLabel</label>
        HTML;

        $out =<<<HTML

                                                        <!-- uifc2/DatePicker.php -->
                                                        <div class="form-group pb-10">
                                                            $out_label
                                                            <div class="input-group date" id="$id" name="$id">
                                                                <input type="text" class="form-control" id="$id" name="$id">
                                                                <span class="input-group-addon">
                                                                    <span class="$icons"></span>
                                                                </span>
                                                            </div>
                                                        </div>
                                                        <!-- uifc2/DatePicker.php -->

        HTML;

        $header_out =<<<HTML
                <!-- Bootstrap Datetimepicker CSS -->
                <link href="/.elm/vendors/bower_components/eonasdan-bootstrap-datetimepicker/build/css/bootstrap-datetimepicker.min.css" rel="stylesheet" type="text/css"/>

        HTML;

        $this->BxPage->setExtraHeaders($header_out);

        $script_out =<<<HTML
                <!-- uifc2/DatePicker.php -->
                <script type="text/javascript" src="/.elm/vendors/bower_components/eonasdan-bootstrap-datetimepicker/build/js/bootstrap-datetimepicker.min.js"></script>

                <script>
                    $(document).ready(function() {
                        "use strict";
                        
                        /* Datetimepicker Init*/
                        $('#$id').datetimepicker({
                                useCurrent: false,
                                format: $format,
                                $viewMode
                                defaultDate: moment($time_stamp, $format),
                                $date_range
                                icons: {
                                        time: "fa fa-clock-o",
                                        date: "fa fa-calendar",
                                        up: "fa fa-arrow-up",
                                        down: "fa fa-arrow-down"
                                    },
                            }).on('dp.show', function() {
                            if($(this).data("DateTimePicker").date() === null)
                                $(this).data("DateTimePicker").date(moment());
                            })$do_submit
                    });
                </script>
                <!-- uifc2/DatePicker.php -->

        HTML;

        $this->BxPage->setExtraFooters($script_out);

        return $out;
    }
}

/*
Copyright (c) 2008-2024 Michael Stauber, SOLARSPEED.NET
Copyright (c) 2008-2024 Team BlueOnyx, BLUEONYX.IT
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>